##########################################  Uncertainty analysis for SOCD  ################################################
rm(list=ls())
wd<-'E:/JZD/TP C stock and turnover/analysis'
setwd(wd)
require('e1071')
require('caret')
#### read in the data sets used ##########
soc.depth<- read.csv('socd_depth.csv') # deep soil cores only, but avergaed by depth
SOCD=read.csv('SamplesiteDatabase_TP C stock.csv',na.strings='NA',header=T) # site data set
   
D10s=read.csv('TPDatabase_Veg.csv',na.strings='NA',header=T)  # grid data set for prediction (2005-15 climate)
n.iter=1000  # the iteration times for the bootstrapping for uncertainty analysis
### get the uncertainty parameters for varying soil depth
### equ<- function(x) -0.0002*x^2+0.1102*x+2.4177
lm1=lm(socd~poly(depth,2,raw=T),data=soc.depth)
sum1=summary(lm1) # check out the modelperformance
err<- predict(lm1,data.frame(depth=D10s$UsedDepth),interval ='confidence')
scaler<- err/predict(lm1,data.frame(depth=30)) # the grid-specific scaler between socd30 and autual socd
######
SOCD=SOCD[SOCD$TP=='y',]
SOCD.svm=SOCD[SOCD$NaNa=='n',1:86] 
######### start the n.iter cycling
socd30.boot<- matrix(NA,nrow(D10s),n.iter) # the predicted 30cm socd using bootstrapped data set
socd3m.boot<- matrix(NA,nrow(D10s),n.iter) # 
samp.func<- function(x) {
  rdata=rnorm(5,pred.bydepth[x,1],sd.bydepth[x])
  rdata=rdata[rdata<pred.bydepth[x,3]& rdata>pred.bydepth[x,2]]
  return(rdata[1])
}
for (i in 1:n.iter) {
  ### first step: predict socd30 using SVM and bootstrapped data set
  rand.site<-sample(1:nrow(SOCD.svm),nrow(SOCD.svm),replace = T)
  rand.data<-SOCD.svm[rand.site,] ### get the random data set
  tobj <- tune.svm(SOCD30~Comp_T_Current+Comp_P_Current+Comp_T_LGM+Comp_P_LGM+Comp_T_MidH+Comp_P_MidH+NDVI
                   +Biome+SandContent+SM+SoilType+CEC+Total_K+Total_P+pH+Elevation+LANDFORM+SLOPE+RELIEF,  
                   data = rand.data, gamma = 10^(-6:-3), cost = 10^(1:2)) # initial model building
  bestGamma <- tobj$best.parameters[[1]]; bestC <- tobj$best.parameters[[2]] # parameter optimization
  model.svm =svm(SOCD30~Comp_T_Current+Comp_P_Current+Comp_T_LGM+Comp_P_LGM+Comp_T_MidH+Comp_P_MidH+AI+NDVI
                 +Biome+SandContent+SM+SoilType+CEC+Total_K+Total_P+pH+Elevation+LANDFORM +SLOPE+RELIEF, 
                 data = rand.data, gamma=bestGamma,cost=bestC,cross = 10) # the final SVM
  sum<-summary(model.svm) # check out the model performance
  ### predict the SOCD30
  socd<-predict(model.svm,D10s) ### predict the socd30 for the random data set
  socd30.boot[,i]<-socd
  ### predict SOCD3m
  pred.bydepth<- socd*scaler # make sure the soil depth matched with the random data set
  sd.bydepth<- (pred.bydepth[,1]-pred.bydepth[,2])/1.96
  sam.bydepth<- sapply(1:length(sd.bydepth), samp.func) 
  socd3m.boot[,i]<-sam.bydepth ## draw random values, which accounted both depth transfer and random sites
}
socd30.bootsum<- colSums(socd30.boot,na.rm=T)
socd3m.bootsum<- colSums(socd3m.boot,na.rm=T)
mean(socd3m.bootsum,na.rm=T) # a first look
mean(socd30.bootsum,na.rm=T) # a first look
#### put the 30cm and 3m soil C stock together
socd.pool<-cbind(socd30.bootsum,socd3m.bootsum)
#### calculate the mean value for each grid 
socd30.bootmean<- rowMeans(socd30.boot,na.rm=T)
socd3m.bootmean<- rowMeans(socd3m.boot,na.rm=T)
socd30.bootci<-quantile(socd30.boot,probs=c(0.025,0.975),na.rm=T)
socd3m.bootci<-quantile(socd3m.boot,probs=c(0.025,0.975),na.rm=T)
socd.grid<- cbind(socd30.bootmean,socd3m.bootmean)
#### Write the results out ####
write.csv(socd.pool,paste(n.iter,'Bootstrapped socd with uncertainty of depth and sites for 30cm and 3m.csv'),row.names=F)
write.csv(cbind(D10s[,c('Longitude','Latitude')],socd.grid),paste(n.iter,'Bootstrapped mean socd for 30cm and 3m.csv'),
          row.names=F)
write.csv(socd30.boot,'socd30.boot.csv')
write.csv(socd3m.boot,'socd3m.boot.csv')
#### save the 1000 times resampling results
same.image('Uncertainty analysis for Tibetan soil C stock.RData') 
####################################################### The End ############################################################